<?php

namespace Drupal\commerce_niubiz\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_niubiz\Constants\CommerceNiubizConstants;
use Drupal\commerce_niubiz\Service\CommerceNiubizService;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides the Niubiz onsite Checkout payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "niubiz",
 *   label = @Translation("Niubiz Payments"),
 *   display_label = @Translation("Niubiz"),
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_niubiz\PluginForm\PaymentMethodAddForm",
 *   },
 * )
 */
class NiubizPaymentGateway extends OffsitePaymentGatewayBase {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * Commerce Niubiz Service.
   *
   * @var \Drupal\commerce_niubiz\Service\CommerceNiubizService
   */
  protected $niubizService;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('datetime.time'),
      $container->get('http_client'),
      $container->get('commerce_niubiz.niubiz')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, PaymentTypeManager $payment_type_manager, PaymentMethodTypeManager $payment_method_type_manager, TimeInterface $time, ClientInterface $http_client, CommerceNiubizService $niubizService) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time);

    $this->httpClient = $http_client;
    $this->niubizService = $niubizService;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'merchant_id' => '',
      'api_key' => '',
      'session_expiry_time' => 20,
      'city_name' => '',
      'subdivisio_code' => '',
      'country_code' => '',
      'postal_code' => '',
      'logo_url' => 'https://www.niubiz.com.pe/assets/images/logo-niubiz.svg',
      'button_color' => '#FF0000',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $apiUser = '';
    if (!empty($this->configuration['api_key'])) {
      $apiKeyText = base64_decode($this->configuration['api_key']);
      $apiUser = explode(':', $apiKeyText)[0];
    }

    $form['merchant_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Merchant ID'),
      '#description' => $this->t('The merchant code generated during the affiliation process.'),
      '#default_value' => $this->configuration['merchant_id'] ?? '',
      '#required' => TRUE,
      '#maxlength' => 9,
    ];

    $form['api_user'] = [
      '#type' => 'textfield',
      '#title' => $this->t('User'),
      '#description' => $this->t('Enter your API username.'),
      '#default_value' => $apiUser,
      '#required' => TRUE,
    ];

    $form['api_password'] = [
      '#type' => 'password',
      '#title' => $this->t('API Password'),
      '#description' => $this->t('Enter your API password.'),
      '#default_value' => '',
      '#required' => TRUE,
    ];

    $form['session_expiry_time'] = [
      '#type' => 'number',
      '#title' => $this->t('Expiry time'),
      '#default_value' => $this->configuration['session_expiry_time'] ?? '',
      '#description' => $this->t('Payment session duration time, expressed in minutes, must be a number between 5 and 20.'),
      '#min' => 5,
      '#max' => 20,
      '#required' => TRUE,
    ];

    $form['city_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('City name'),
      '#description' => $this->t("Shipping City/District or the Merchant's."),
      '#default_value' => $this->configuration['city_name'] ?? '',
      '#required' => TRUE,
    ];

    $form['subdivisio_code'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Subdivision code'),
      '#description' => $this->t("Shipping Department/Province or the Merchant's. It must be sent in ISO 3166-2 format, for example (LMA)."),
      '#default_value' => $this->configuration['subdivisio_code'] ?? '',
      '#required' => TRUE,
    ];

    $form['country_code'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Country code'),
      '#description' => $this->t("Shipping Country or the Merchant's. It must be sent in ISO 3166-1 alpha-3 format, for example (PER)."),
      '#default_value' => $this->configuration['country_code'] ?? '',
      '#required' => TRUE,
    ];

    $form['postal_code'] = [
      '#type' => 'number',
      '#title' => $this->t('Postal code'),
      '#default_value' => $this->configuration['postal_code'] ?? '',
      '#description' => $this->t("Shipping Postal Code or the Merchant's, for example (15023)."),
      '#required' => TRUE,
    ];

    $form['widget_options'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Widget Options'),
    ];

    $form['widget_options']['logo_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Store Logo URL'),
      '#default_value' => $this->configuration['logo_url'],
    ];

    $form['widget_options']['button_color'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Button color'),
      '#default_value' => $this->configuration['button_color'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $apiKey = base64_encode($values['api_user'] . ':' . $values['api_password']);
      $this->configuration['merchant_id'] = $values['merchant_id'];
      $this->configuration['api_key'] = $apiKey;
      $this->configuration['session_expiry_time'] = $values['session_expiry_time'];
      $this->configuration['city_name'] = $values['city_name'];
      $this->configuration['subdivisio_code'] = $values['subdivisio_code'];
      $this->configuration['country_code'] = $values['country_code'];
      $this->configuration['postal_code'] = $values['postal_code'];
      $this->configuration['logo_url'] = $values['widget_options']['logo_url'];
      $this->configuration['button_color'] = $values['widget_options']['button_color'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    parent::onReturn($order, $request);

    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
    $authorization = $this->generateAuthorization($order, $request);
    $dataResponse = $authorization['dataMap'] ?? $authorization['data'];
    $niubizOrderNumber = $this->niubizService->getNiubizOrderNumber($order);
    if ($dataResponse['STATUS'] === 'Authorized') {
      $payment_response_info = [
        'niubiz_order_number' => $niubizOrderNumber,
        'card' => $dataResponse['CARD'] ?? '',
        'brand' => $dataResponse['BRAND'] ?? '',
      ];

      $payment = $payment_storage->create([
        'state' => 'completed',
        'amount' => $order->getTotalPrice(),
        'payment_gateway' => $this->parentEntity->id(),
        'order_id' => $order->id(),
        'remote_id' => $dataResponse['TRANSACTION_ID'] . ' (Order: ' . $niubizOrderNumber . ')',
        'remote_state' => $dataResponse['STATUS'],
      ]);

      $payment->save();

    }
    else {
      $payment_response_info = [
        'niubiz_order_number' => $niubizOrderNumber,
        'card' => $dataResponse['CARD'] ?? '',
        'brand' => $dataResponse['BRAND'] ?? '',
        'description' => $dataResponse['ACTION_DESCRIPTION'] ?? '',
      ];

      $this->messenger()->addError($this->t('Order number: @niubiz_order_number', ['@niubiz_order_number' => $niubizOrderNumber]));
      $this->messenger()->addError($this->t('Date: @date', ['@date' => date("Y/m/d H:i:s", $order->getPlacedTime())]));
      $this->messenger()->addError($this->t('Message: @message', ['@message' => $dataResponse['ACTION_DESCRIPTION']]));
    }

    // @todo This field is essential for storing additional transaction information displayed upon completing the purchase. Note that this module does not create any fields automatically; refer to the README.md file for instructions on how to create the required fields.
    // A stable solution is yet to be developed.
    if ($order->hasField('field_payment_response_info')) {
      $existing_payment_response_info = $this->niubizService->getPaymentResponseInfo($order);
      $field_definition = $order->getFieldDefinition('field_payment_response_info')->getFieldStorageDefinition();

      if ($field_definition->isMultiple()) {
        $existing_payment_response_info[] = ['value' => json_encode($payment_response_info)];
        $order->set('field_payment_response_info', $existing_payment_response_info);
        $order->save();
      }
    }

    if ($dataResponse['STATUS'] !== 'Authorized') {
      throw new PaymentGatewayException(
        $dataResponse['ACTION_CODE'] . ' - ' . $dataResponse['ACTION_DESCRIPTION']
      );
    }
  }

  /**
   * Generates an authorization for a transaction via the Niubiz API.
   *
   * @param object $order
   *   The order object containing details about the purchase.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object containing the transaction token and other data.
   *
   * @return array|null
   *   Niubiz API response as an associative array, or NULL on failure.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   *   If there is an issue with the HTTP request.
   */
  private function generateAuthorization($order, $request) {
    // Validate the order and request objects.
    if (!$order || !$request->get('transactionToken')) {
      throw new \InvalidArgumentException('Invalid order or transaction token.');
    }

    $token = $this->niubizService->generateSecurityToken($this->configuration);
    $orderAmount = explode(' ', $order->getTotalPrice());
    if (count($orderAmount) !== 2) {
      throw new \UnexpectedValueException('Order total price must include both amount and currency.');
    }

    $address = $order->getBillingProfile()->get('address');
    $body = [
      'channel' => 'web',
      'captureType' => 'manual',
      'countable' => TRUE,
      'order' => [
        'tokenId' => $request->get('transactionToken'),
        'purchaseNumber' => $this->niubizService->getNiubizOrderNumber($order),
        'amount' => $orderAmount[0],
        'currency' => $orderAmount[1],
      ],
      'dataMap' => [
        'urlAddress' => Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString(),
        'partnerIdCode' => "",
        'serviceLocationCityName' => $this->configuration['city_name'], //$address->locality,
        'serviceLocationCountrySubdivisionCode' => $this->configuration['subdivisio_code'], //$address->administrative_area,
        'serviceLocationCountryCode' => $this->configuration['country_code'], //$address->country_code,
        'serviceLocationPostalCode' => $this->configuration['postal_code'], //$address->postal_code,
      ],
    ];

    $options = [
      'method' => 'POST',
      'headers' => [
        'Content-Type' => 'application/json',
        'Authorization' => $token,
      ],
      'body' => json_encode($body),
    ];

    $baseUrl = $this->configuration['mode'] === 'test'
      ? CommerceNiubizConstants::SANDBOX_BASE_URL
      : CommerceNiubizConstants::PRODUCTION_BASE_URL;
    $merchantId = $this->configuration['merchant_id'];
    $apiUrl = $baseUrl . CommerceNiubizConstants::AUTHORIZATION_API . $merchantId;

    try {
      $response = $this->httpClient->request('POST', $apiUrl, $options);
      return json_decode($response->getBody()->getContents(), TRUE);
    }
    catch (\Exception $e) {
      // Log the exception for debugging purposes.
      \Drupal::logger('commerce_niubiz')->error('Authorization request failed: @message', [
        '@message' => $e->getMessage(),
      ]);

      // Attempt to extract the response from the exception, if available.
      if ($e instanceof RequestException && $e->hasResponse()) {
        return json_decode($e->getResponse()->getBody()->getContents(), TRUE);
      }

      // Re-throw the exception if no meaningful response is available.
      throw $e;
    }
  }

}
