<?php

namespace Drupal\commerce_niubiz\PluginForm;

use Drupal\commerce_niubiz\Constants\CommerceNiubizConstants;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Url;
use Drupal\user\Entity\User;

/**
 * Custom payment form.
 */
class PaymentMethodAddForm extends PaymentOffsiteForm {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->getEntity();
    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $paymentGatewayPlugin */
    $paymentGatewayPlugin = $payment->getPaymentGateway()->getPlugin();
    $paymentGatewayConfig = $paymentGatewayPlugin->getConfiguration();
    $sessionKey = $this->generateSessionKey($paymentGatewayConfig);
    $niubizService = \Drupal::service('commerce_niubiz.niubiz');
    $niubizOrderNumber = $niubizService->getNiubizOrderNumber($payment->getOrder());

    $data = [
      'sessionKey' => $sessionKey,
      'version' => 'v1',
      'merchantId' => $paymentGatewayConfig['merchant_id'],
      'logoUrl' => $paymentGatewayConfig['logo_url'],
      'payButtonColor' => $paymentGatewayConfig['button_color'],
      'order_id' => $niubizOrderNumber,
      'currency' => $payment->getAmount()->getCurrencyCode(),
      'amount' => $payment->getAmount()->getNumber(),
      'continueUrl' => $this->buildReturnUrl($payment->getOrder()->id()),
      'cancelUrl' => $this->buildCancelUrl($payment->getOrder()->id()),
      'customerEmail' => $payment->getOrder()->getEmail(),
      'mode' => $paymentGatewayConfig['mode'],
      'expirationminutes' => $paymentGatewayConfig['session_expiry_time'],
    ];

    $form['#attached']['library'][] = ($paymentGatewayConfig['mode'] === 'test') ? 'commerce_niubiz/niubiz_sandbox' : 'commerce_niubiz/niubiz_production';
    $form['#attached']['library'][] = 'commerce_niubiz/niubiz';
    $form = $this->buildRedirectForm($form, $form_state, '', $data, '');
    $form['#attached']['drupalSettings']['niubiz']['transactionData'] = json_encode($data);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function buildRedirectForm(array $form, FormStateInterface $form_state, $redirect_url, array $data, $redirect_method = PaymentOffsiteForm::REDIRECT_GET) {
    $helpMessage = $this->t('Please wait, the payment gateway is loading.');

    $form['commerce_message'] = [
      '#markup' => '<div class="checkout-help">' . $helpMessage . '</div>',
      '#weight' => -10,
      '#process' => [
        [get_class($this), 'processRedirectForm'],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public static function processRedirectForm(array $element, FormStateInterface $form_state, array &$complete_form) {
    $complete_form['#attributes']['class'][] = 'payment-redirect-form';
    unset($element['#action']);
    // The form actions are hidden by default, but needed in this case.
    $complete_form['actions']['#access'] = TRUE;
    foreach (Element::children($complete_form['actions']) as $element_name) {
      $complete_form['actions'][$element_name]['#access'] = TRUE;
    }

    return $element;
  }

  /**
   * Generates a session key for the payment gateway.
   *
   * This method sends a POST request to the payment gateway's API.
   *
   * @param array $paymentGatewayConfig
   *   Configuration array for the payment gateway.
   *
   * @return string|null
   *   The session key if successful, or null if an error occurs.
   *
   * @throws \Drupal\commerce_niubiz\PaymentGatewayException
   *   If the API call fails.
   */
  private function generateSessionKey(array $paymentGatewayConfig) {
    $niubizService = \Drupal::service('commerce_niubiz.niubiz');
    $payment = $this->getEntity();
    $antifraud = $this->buildAntifraudData();
    $token = $niubizService->generateSecurityToken($paymentGatewayConfig);
    $address = $payment->getOrder()->getBillingProfile()->get('address');

    // @todo This field is required and is currently created manually (see README.md).
    // A stable solution is yet to be developed.
    $phone = $payment->getOrder()->getBillingProfile()->get('field_customer_phone')->value;

    $body = [
      'amount' => $payment->getAmount()->getNumber(),
      'antifraud' => $antifraud,
      'channel' => 'web',
      'dataMap' => [
        'cardholderCity' => $address->locality,
        'cardholderCountry' => $address->country_code,
        'cardholderAddress' => $address->address_line1 . ', ' . $address->address_line2,
        'cardholderPostalCode' => $address->postal_code,
        'cardholderState' => $address->administrative_area,
        'cardholderPhoneNumber' => $phone,
      ],
    ];

    $options = [
      'method' => 'POST',
      'headers' => [
        'Content-Type' => 'application/json',
        'Authorization' => $token,
      ],
      'body' => json_encode($body),
    ];

    try {
      $client = \Drupal::httpClient();
      $baseUrl = $paymentGatewayConfig['mode'] === 'test'
      ? CommerceNiubizConstants::SANDBOX_BASE_URL
      : CommerceNiubizConstants::PRODUCTION_BASE_URL;
      $response = $client->request('POST', $baseUrl . CommerceNiubizConstants::SESSION_API . $paymentGatewayConfig['merchant_id'], $options);

      if ($response->getStatusCode() == 200) {
        $jsonResponse = json_decode($response->getBody(), TRUE);
        return $jsonResponse['sessionKey'];
      }

      return NULL;
    }
    catch (\Exception $e) {
      $response = $e->getResponse();
      throw new PaymentGatewayException("API Session ERROR: " . $response->getBody()->getContents());
    }
  }

  /**
   * Builds antifraud data for the payment request.
   *
   * @return array
   *   An array of antifraud data.
   */
  protected function buildAntifraudData() {
    $payment = $this->getEntity();
    $daysRegistered = 1;

    if ($payment->getOrder()->getCustomerId()) {
      $user = User::load($payment->getOrder()->getCustomerId());
      if ($user) {
        $created = $user->getCreatedTime();
        $currentTime = time();
        $daysRegistered = floor(($currentTime - $created) / 86400);
      }
    }

    return [
      'clientIp' => $payment->getOrder()->getIpAddress(),
      'merchantDefineData' => [
        'MDD4' => $payment->getOrder()->getEmail(),
        'MDD32' => $payment->getOrder()->getEmail(),
        'MDD75' => $payment->getOrder()->getCustomerId() == '0' ? 'Invitado' : 'Registrado',
        'MDD77' => $daysRegistered,
      ],
    ];
  }

  /**
   * Builds the URL to the "return" page.
   *
   * @return \Drupal\Core\Url
   *   The "return" page URL.
   */
  protected function buildReturnUrl($order_id) {
    return Url::fromRoute('commerce_payment.checkout.return', [
      'commerce_order' => $order_id,
      'step' => 'payment',
    ])->toString(TRUE)->getGeneratedUrl();
  }

  /**
   * Builds the URL to the "cancel" page.
   *
   * @return \Drupal\Core\Url
   *   The "cancel" page URL.
   */
  protected function buildCancelUrl($order_id) {
    return Url::fromRoute('commerce_payment.checkout.cancel', [
      'commerce_order' => $order_id,
      'step' => 'payment',
    ])->toString(TRUE)->getGeneratedUrl();
  }

}
