<?php

namespace Drupal\commerce_niubiz\Service;

use Drupal\commerce_niubiz\Constants\CommerceNiubizConstants;
use Drupal\Core\Entity\EntityInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;

/**
 * Commerce Niubiz utility service.
 */
class CommerceNiubizService {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * CommerceNiubizService constructor.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   */
  public function __construct(ClientInterface $http_client) {
    $this->httpClient = $http_client;
  }

  /**
   * Generates a security token using the payment gateway configuration.
   *
   * @param array $paymentGatewayConfig
   *   The payment gateway configuration array.
   *
   * @return string|null
   *   The security token or NULL on failure.
   *
   * @throws \Drupal\commerce_payment\Exception\PaymentGatewayException
   *   If the token generation request fails.
   */
  public function generateSecurityToken(array $paymentGatewayConfig) {
    if (empty($paymentGatewayConfig['api_key'])) {
      throw new PaymentGatewayException('The API key is missing from the payment gateway configuration. Please provide a valid API key.');
    }

    $options = [
      'headers' => [
        'Authorization' => 'Basic ' . $paymentGatewayConfig['api_key'],
      ],
    ];

    try {
      $baseUrl = ($paymentGatewayConfig['mode'] === 'test') ? CommerceNiubizConstants::SANDBOX_BASE_URL : CommerceNiubizConstants::PRODUCTION_BASE_URL;
      $response = $this->httpClient->post($baseUrl . CommerceNiubizConstants::SECURITY_API, $options);
      if ($response->getStatusCode() === 201) {
        return $response->getBody()->getContents();
      }

      return NULL;
    }
    catch (RequestException $e) {
      throw new PaymentGatewayException('API Security Request Error: ' . $e->getMessage());
    }
    catch (\Exception $e) {
      throw new PaymentGatewayException('API Security ERROR: ' . $e->getMessage());
    }
  }

  /**
   * Generates a unique order number for the payment process based on the entity's ID and the number of previous attempts.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity containing the payment response field.
   *
   * @return string
   *   A string composed of the entity ID and a zero-padded two-digit attempt number
   *   (e.g., '12301', '12302', ..., '12399').
   */
  public function getNiubizOrderNumber(EntityInterface $entity) {
    $attempt_number = 1;
    $payment_response_info = $this->getPaymentResponseInfo($entity);

    if (!empty($payment_response_info)) {
      $attempt_number = min(count($payment_response_info) + 1, 99);
    }

    $attempt_number_padded = str_pad((string) $attempt_number, 2, '0', STR_PAD_LEFT);

    return $entity->id() . $attempt_number_padded;
  }

  /**
   * Retrieves the payment response information for a given entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity containing the payment response field.
   *
   * @return array
   *   An array containing the existing payment response data if available, or an empty array if no data is found.
   */
  public function getPaymentResponseInfo(EntityInterface $entity): array {
    $payment_response_info = [];

    if ($entity->hasField('field_payment_response_info')) {
      $payment_response_info = $entity->get('field_payment_response_info')->getValue();
    }

    return $payment_response_info;
  }

}
